﻿(function () {
    angular.module('app').factory('accountManager', ['httpService', 'session', function (httpService, session) {

        var module = {
            userInfo: null,
            register: function (args) {
                return httpService.post({
                    url: 'api/account/register',
                    data: args
                });
            },
            login: function (email, password) {
                var self = this;
                var url = 'Token';
                var args = 'grant_type=password&username=' + email + '&password=' + password;
                return httpService.post({
                    url: url,
                    data: args,
                    headers: {
                        'Content-Type': 'application/x-www-form-urlencoded'
                    }
                })
                .success(function (data) {
                    httpService.setToken(data.access_token);
                });
            },
            logout: function () {
                var url = 'api/account/logout/';
                return httpService.post({
                    url: url
                })
                .success(function () {
                    _destroySession();
                })
                .error(function () {
                    _destroySession()
                });
            },
            getUserInfo: function () {
                var self = this;

                return httpService.get({
                    url: 'api/account/userInfo'
                })
                .success(function (data) {
                    self.userInfo = data;
                    //console.info('userInfo', self.userInfo);
                })
                .error(function (error) {
                    self.userInfo = null;
                    httpService.unsetToken();
                    console.error(error);
                });
            },
            updateSexualInterests: function (args) {
                return httpService.patch({
                    url: 'api/account/sexualInterests',
                    data: args
                });
            },
            toggleIntersetedInFemales: function () {

                var self = this;

                return httpService.patch({
                    url: 'api/account/toggleIntersetedInFemales'
                })
                .success(function () {
                    self.getUserInfo();
                });
            },
            toggleIntersetedInMales: function () {

                var self = this;

                return httpService.patch({
                    url: 'api/account/toggleIntersetedInMales'
                })
                .success(function () {
                    self.getUserInfo();
                });
            },
            updateDisplayName: function (displayName) {

                var self = this;

                return httpService.patch({
                    url: 'api/account/displayName',
                    data: {
                        displayName: displayName
                    }
                })
                .success(function () {
                    self.getUserInfo();
                });
            },
            getClientsIds: function () {
                var url = 'api/account/ClientsIds/';
                return httpService.get({
                    url: url
                });
            },
            getLocalToken: function (provider, token) {
                var self = this;
                var url = 'api/account/LocalToken/?provider=' + provider + '&token=' + token;
                return httpService.get({
                    url: url
                })
                .success(function (data) {
                    httpService.setToken(data.access_token);
                });
            },
            registerExternal: function (args) {
                var self = this;
                var url = 'api/account/registerExternal/';
                return httpService.post({
                    url: url,
                    data: args
                })
                .success(function (data) {
                    httpService.setToken(data.access_token);
                });
            },
            updateName: function (args) {
                return httpService.post({
                    url: 'api/account/updateName/',
                    data: args
                });
            },
            updateProfilePhoto: function (imageData) {
                var args = {
                    imageData: imageData
                };
                console.info(args);
                return httpService.patch({
                    url: 'api/account/profilePhoto/',
                    data: args
                });
            },
            setSawFirstPage: function () {
                return httpService.patch({
                    url: 'api/account/sawFirstPage/'
                });
            },
            setLastLocation: function (args) {
                return httpService.post({
                    url: 'api/account/lastLocation',
                    data: args
                });
            }
        }

        return module;

        function _destroySession() {
            module.userInfo = null;
            httpService.unsetToken();
            session.clear();
        }

    }]);
}());