﻿(function () {
    angular.module('osmos')
        .factory('httpService', ['$q', '$http', '$window',
            function ($q, $http, $window) {

                var osmos = angular.injector(['osmos']);

                var tokenName = osmos.has('tokenName') ? osmos.get('tokenName') : 'osHttpAccessToken';
                var sessionIdKeyName = osmos.has('sessionIdKeyName') ? osmos.get('sessionIdKeyName') : 'session-id';
                var baseUrl = osmos.has('baseUrl') ? osmos.get('baseUrl') : '';
                var logger = osmos.has('logger') ? osmos.get('logger') : null;

                var module = {
                    currentRequests: [],
                    execute: function (args) {
                        var self = this;

                        var options = {};
                        options.url = baseUrl + args.url;
                        options.method = args.method;
                        options.data = args.data;
                        options.params = args.params,
                        options.paramSerializer = '$httpParamSerializerJQLike';

                        var showLoader = args.showLoader || true;
                        var showToast = args.showToast || {
                            success: false,
                            error: false
                        };

                        var req = new Request(showLoader);
                        self.currentRequests.push(req);

                        var securityHeaders = _getDefaultHeaders();

                        if (args.headers) {
                            options.headers = _mergeObjects(args.headers, securityHeaders);
                        }
                        else {
                            options.headers = securityHeaders;
                        }

                        if (args.transformResponse) {
                            options.transformResponse = args.transformResponse;
                        }

                        self.showLoader = showLoader;

                        return $http(options)
                            .success(function (data, status, headers) {
                                if (showToast.success && logger) logger.success('operation success.');

                                _updateShowLoader(req);
                            })
                            .error(function (data, status, headers) {
                                if (showToast.error && logger) {

                                    if (status === 400 && data.message) {
                                        logger.error(data.message);
                                    }
                                    else {
                                        logger.error('operation error.');
                                    }
                                }
                                _updateShowLoader(req);
                            });
                    },
                    get: function (args) {
                        var self = this;
                        args.method = 'get';
                        return self.execute(args);
                    },
                    post: function (args) {
                        var self = this;

                        args.method = 'post';
                        args.showToast = args.showToast || {
                            success: false,
                            error: true
                        };

                        return self.execute(args);
                    },
                    put: function (args) {
                        var self = this;

                        args.method = 'put';
                        args.showToast = args.showToast || {
                            success: false,
                            error: true
                        };

                        return self.execute(args);
                    },
                    patch: function (args) {
                        var self = this;

                        args.method = 'patch';
                        args.showToast = args.showToast || {
                            success: false,
                            error: true
                        };

                        return self.execute(args);
                    },
                    delete: function (args) {
                        var self = this;

                        args.method = 'delete';
                        args.showToast = args.showToast || {
                            success: false,
                            error: true
                        };

                        return self.execute(args);
                    },
                    setToken: function (accessToken) {
                        $window.localStorage[tokenName] = accessToken;
                    },
                    setSessionId: function (sessionId) {
                        $window.localStorage[sessionIdKeyName] = sessionId;
                    },
                    unsetToken: function () {
                        $window.localStorage[tokenName] = null;
                    },
                    showLoader: false,
                    tokenName: tokenName,
                    sessionIdKeyName: sessionIdKeyName,
                    createEmptyPromise: function (value) {
                        var promise = $q.resolve(value);

                        promise.success = function (fn) {
                            promise.then(function (response) {
                                fn(response);
                            }, null);
                            return promise;
                        };

                        promise.error = function (fn) {
                            promise.then(null, function (response) {
                                fn(response);
                            });
                            return promise;
                        };

                        return promise;
                    }
                };

                return module;

                function Request(showLoader) {
                    this.showLoader = showLoader;
                }

                function _getDefaultHeaders() {
                    var token = $window.localStorage[tokenName];
                    var sessionId = $window.localStorage[sessionIdKeyName];

                    var result = {};
                    
                    if (token) result['Authorization'] = 'Bearer ' + token;
                    if (sessionId) result[sessionIdKeyName] = sessionId;

                    return result
                }

                function _mergeObjects(obj1, obj2) {
                    var obj3 = {};
                    var attrname;
                    for (attrname in obj1) { obj3[attrname] = obj1[attrname]; }
                    for (attrname in obj2) { obj3[attrname] = obj2[attrname]; }
                    return obj3;
                }

                function _updateShowLoader(req) {
                    var index = module.currentRequests.indexOf(req);
                    if (index > -1) {
                        module.currentRequests.splice(index, 1);
                        module.showLoader = _hasToShowLoader(module.currentRequests);
                    }
                }

                function _hasToShowLoader(currentRequests) {
                    for (var k = 0; k < currentRequests.length; k++) {
                        if (currentRequests[k].showLoader) return true;
                    }
                    return false;
                }
            }]);
}());