﻿(function () {
    angular.module('app').factory('signalr', ['$state', '$rootScope', '$ionicLoading', '$timeout', 'rtServer', 'accountManager', 'analytics', 'messagesManager',
      function ($state, $rootScope, $ionicLoading, $timeout, rtServer, accountManager, analytics, messagesManager) {

          return {
              connected: false,
              session: null,
              connection: null,
              appHubProxy: null,
              sessionsCount: 0,
              init: function () {
                  var self = this;

                  if (!self.connection || self.connection.state == 4) {
                      self.connection = $.hubConnection(rtServer + 'signalr', { useDefaultPath: false });
                      self.connection.qs = {
                          userId: accountManager.userInfo.id
                      };

                      self.connection.error(function (error) {
                          console.error('SignalR error: ' + error)
                      });

                      self.connection.logging = false;

                      self.appHubProxy = self.connection.createHubProxy('appHub');

                      self.appHubProxy.on('ping', function () {
                          self.appHubProxy.invoke('pong');
                      });

                      self.appHubProxy.on('disconnect', function () {
                          self.connection.stop();
                      });

                      self.appHubProxy.on('pong', function (data) {
                          console.log('pong', data);
                      });

                      self.appHubProxy.on('messageReceivedFrom', function (friendId) {
                          if ($state.current.name !== 'app.textChat')
                             messagesManager.getHasUnseenMessages();
                      });

                      self.appHubProxy.on('hello', function (data) {
                          console.log(data);
                      });

                      self.appHubProxy.on('log', function (data) {
                          console.info('signalr log : ', data);

                          if (data && data.state == 'Error' && data.message) {
                              $ionicLoading.show({
                                  template: data.message
                              });

                              $timeout(function () {
                                  $ionicLoading.hide();
                              }, 1000);
                          }
                      });

                      self.appHubProxy.on('sessionHasChanged', function (session) {
                          self.session = session;
                          $rootScope.$apply();
                          //console.info('self sessionHasChanged', session);
                      });

                      self.appHubProxy.on('sessionsCountChanged', function (sessionsCount) {
                          self.sessionsCount = sessionsCount;
                          $rootScope.$apply();
                      });

                      self.appHubProxy.on('nexted', function () {
                          $ionicLoading.show({
                              template: 'You have been nexted...'
                          });

                          $timeout(function () {
                              $ionicLoading.hide();
                          }, 1500);
                      });

                      self.connection.start()
                        .done(function () {
                            self.connected = true;

                            self.connection.disconnected(function () {
                                console.log('Disconnected from signalr.');
                                self.connected = false;
                                self.appHubProxy = null;
                                self.connection = null;
                                self.session = null;

                                analytics.event('disconnectedFromSR');
                            });

                            console.info('self.connection', self.connection);

                            $rootScope.$apply();

                            self.appHubProxy.invoke('ping');
                            console.log('ping');

                            analytics.event('connectedToSR');
                        })
                        .fail(function () {
                            self.appHubProxy = null;
                            self.connection = null;

                            //$rootScope.$apply();
                        });
                  }
              },
              stop: function () {
                  var self = this;
                  if (self.connection && self.connected) {
                      self.connection.stop();
                  }
              },
              invoke: function () {
                  var self = this;
                  if (self.appHubProxy && self.connected) {
                      self.appHubProxy.invoke.apply(self.appHubProxy, arguments);
                  }
              },
              on: function (name, callback) {
                  var self = this;
                  if (self.appHubProxy && self.connected) {
                      self.appHubProxy.off(name);
                      self.appHubProxy.on(name, callback);
                  }
              }
          };
      }]);
}());
