﻿(function () {
    angular.module('app').controller('loginCtrl', ['$state', '$ionicLoading', '$cordovaOauth', 'accountManager', 'qbManager', 'signalr', 'analytics', 'dev', 'facebookManager', 'gaManager',
        function ($state, $ionicLoading, $cordovaOauth, accountManager, qbManager, signalr, analytics, dev, facebookManager, gaManager) {
            var vm = this;

            analytics.pageActivated('login');

            vm.connectingToFB = false;


            vm.dev = dev;


            vm.user = {
                email: '',
                password: ''
            };

            vm.error = null;
            vm.data = null;

            vm.login = function () {
                vm.error = null;
                vm.data = null;
                $ionicLoading.show({
                    template: 'Loading...'
                });
                accountManager.login(vm.user.email, vm.user.password)
                .success(function (data) {
                    console.log(data);
                    vm.data = JSON.stringify(data);
                    analytics.action('login');
                    getUserInfo();
                })
                .error(function (error) {
                    console.error(error);
                    if (error.error_description) {
                        vm.error = error.error_description;
                        return;
                    }
                    vm.error = JSON.stringify(error);
                })
                .finally(function () {
                    $ionicLoading.hide();
                });
            }

            vm.facebook = function () {
                if (!ionic.Platform.is('browser'))
                    $cordovaOauth.facebook(facebookManager.appId, [facebookManager.allScopes]).then(function (result) {
                        if (result['access_token']) {
                            processAuth("Facebook", result.access_token)
                                .then(function () {
                                    facebookManager.getUserInfo(result.access_token);
                                    gaManager.fbLogin();
                                });
                            
                        } else {
                            console.error("access token not found");
                        };

                    }, function (error) {
                        console.log("Error -> " + error);
                    });
                else {
                    statusChangeCallback();
                }
            }

            getUserInfo();

            function statusChangeCallback() {

                vm.connectingToFB = true;                


                if (facebookManager.loginStatus.status === 'connected') {
                    if (facebookManager.loginStatus.authResponse['accessToken']) {
                        
                        processAuth("Facebook", facebookManager.loginStatus.authResponse.accessToken)
                            .then(function () {
                                facebookManager.getUserInfo();
                                gaManager.fbLogin();
                            });
                    } 
                }
                else {

                    facebookManager.login().then(function (response) {
                        if (response.status === 'connected') {
                            if (response.authResponse['accessToken']) {
                                processAuth("Facebook", response.authResponse.accessToken)
                                    .then(function () {
                                        facebookManager.getUserInfo();
                                        gaManager.fbLogin();
                                    });
                            }
                        };
                    }).catch(function (e) {
                        console.error('Login error.', e)
                    });
                }

                
            }

            function processAuth(provider, accessToken) {
                console.log("process");
                console.log('provider', provider);
                console.log('accessToken', accessToken);
                return accountManager.getLocalToken(provider, accessToken)
                 .then(function successCallback(response) {
                     console.log(response);
                     accountManager.getUserInfo()
                     .finally(function () {
                         analytics.action('login FB');
                         $state.go('zero');
                     });
                 }, function errorCallback(response) {
                     console.log('error', response);
                     switch (response.status) {
                         //case 404:
                         //    $state.go('registerExternal', { provider: provider, token: accessToken });
                         //    break;
                         case 403:
                             console.error(response.data);
                             break;
                     }
                 });
            }

            function getUserInfo() {
                accountManager.getUserInfo()
                    .success(function (data) {
                        console.log(data);
                        vm.data = JSON.stringify(data);

                        $state.go('app.zero');
                    })
                    .error(function (error) {
                        console.error(error);
                        vm.error = JSON.stringify(error);
                    })
                    .finally(function () {
                        $ionicLoading.hide();
                    });
            }

        }]);
}());
