(function () {
    angular.module('app').controller('friendsCtrl', ['$rootScope', '$state', '$ionicPopup', 'accountManager', 'friendshipsManager', 'signalr', 'analytics','messagesManager',
        function ($rootScope, $state, $ionicPopup, accountManager, friendshipsManager, signalr, analytics, messagesManager) {
            var vm = this;

            analytics.pageActivated('friends');

            vm.userInfo = null;
            var friendships = [];

            vm.friendRequests = [];
            vm.friends = [];


            signalr.on('messageReceivedFrom', function (friendId) {
                messagesManager.getHasUnseenMessages();
                getUnseenMessagesUserIds();
            });

            signalr.on('friendIsOnline', function (userId) {
                getFriendSessions();
            });

            signalr.on('friendIsOffline', function (userId) {
                getFriendSessions();
            });

            signalr.on('friendsOnline', function (onlineFriendIds) {
                var onlineFriendships = [];

                for (var k in vm.friends) {
                    var onlineFriendId = _.find(onlineFriendIds, function (id) {
                        return vm.friends[k].from.id === id || vm.friends[k].to.id === id;
                    });

                    if (onlineFriendId) vm.friends[k].online = true;
                    else vm.friends[k].online = false;
                }

                $rootScope.$apply();

            });

            vm.startTextChat = function (friendshipId) {
                $state.go('app.textChat', { friendshipId: friendshipId });

                analytics.action('start text chat', [{
                    id: friendshipId
                }]);

            }

            vm.acceptFriendRequest = function (friendRequestId) {
                var confirmPopup = $ionicPopup.confirm({
                    title: 'Accept Friend'
                });

                confirmPopup.then(function (res) {
                    if (res) {
                        friendshipsManager.accept(friendRequestId)
                        .success(function () {
                            getFriendships();
                        });
                    }
                });

                analytics.action('accept friend', [{
                    id: friendshipId
                }]);
            }

            vm.rejectFriendRequest = function (friendRequestId) {
                var confirmPopup = $ionicPopup.confirm({
                    title: 'Reject Friend'
                });

                confirmPopup.then(function (res) {
                    if (res) {
                        friendshipsManager.reject(friendRequestId)
                        .success(function () {
                            getFriendships();
                        });
                    }
                });

                analytics.action('reject friend', [{
                    id: friendshipId
                }]);
            }

            accountManager.getUserInfo()
            .success(function (data) {
                vm.userInfo = accountManager.userInfo;

                getFriendships();

            })
            .error(function () {
                $state.go('login');
            });

            function getFriendships() {
                friendshipsManager.getAll()
                .success(function (data) {

                    console.info(data);

                    friendships = _.map(data, function (item) {
                        if (item.createdDate) {
                            item.createdDate = moment(item.createdDate).format('dddd MMM Do YYYY HH:mm');
                        }

                        return item;
                    });

                    vm.friends = _.filter(friendships, function (item) {
                        return (item.to.id === vm.userInfo.id || item.from.id === vm.userInfo.id) && item.state === 'Accepted';
                    });

                    vm.friendRequests = _.filter(friendships, function (item) {
                        return item.to.id === vm.userInfo.id && item.state === 'Pending';
                    });

                    console.info('vm.friends', vm.friends);
                    console.info('vm.friendRequests', vm.friendRequests);

                    vm.unseen = _.filter(vm.friendRequests, function (item) {
                        return !item.seen;
                    });

                    vm.unseen = _.map(vm.unseen, function (item) {
                        return item.id;
                    });

                    if (vm.unseen && vm.unseen.lenght) {
                        friendshipsManager.seen({
                            ids: vm.unseen
                        });
                    }

                    getFriendSessions();

                    getUnseenMessagesUserIds();

                });
            }

            function getFriendSessions() {
                signalr.invoke('getFriendSessions');
            }

            function getUnseenMessagesUserIds() {
                messagesManager.getUnseenMessagesUserIds()
                .success(function (data) {
                    console.info('getUnseenMessagesUserIds', data);

                    var tmp = [];

                    for (var k in vm.friends) {
                        var friend = vm.friends[k];
                        friend.hasMessages = false;

                        if (friend.to && friend.to.id && friend.from && friend.from.id) {
                            for (var l in data) {
                                if (friend.to.id === data[l] || friend.from.id === data[l]) {
                                    friend.hasMessages = true;
                                    break;
                                }
                            }
                        }

                        tmp.push(friend);
                    }

                    tmp = _.orderBy(tmp, function (item) {
                        return item.hasMessages;
                    }, 'desc');

                    vm.friends = angular.copy(tmp);

                    tmp = [];
                    tmp.length = 0;
                    console.log('friend.hasMessages', friend.hasMessages);
                })
                .error(function (error) {
                    console.error('getUnseenMessagesUserIds', error);
                });
            }

        }]);
}());
