(function () {
    angular.module('app').controller('textChatCtrl', ['$rootScope', '$state', '$stateParams', '$ionicScrollDelegate', 'accountManager', 'messagesManager', 'signalr', 'analytics', 'friendshipsManager', 'gaManager',
    function ($rootScope, $state, $stateParams, $ionicScrollDelegate, accountManager, messagesManager, signalr, analytics, friendshipsManager, gaManager) {
        var vm = this;

        analytics.pageActivated('textChat');

        console.info('$ionicScrollDelegate', $ionicScrollDelegate);

        vm.userInfo = null;

        signalr.on('messageReceivedFrom', function (friendId) {
            if ($state.current.name === 'app.textChat') {
                getMessages();
            } else {
                messagesManager.getHasUnseenMessages();
            }
            
        });

        vm.text = '';
        vm.messages = [];



        vm.exitTextChat = function () {
            $state.go('app.friends');
            analytics.action('exit chat');
        }

        vm.sendMessage = function () {
            if (vm.text && vm.text.length > 0) {
                messagesManager.sendMessage($stateParams.friendshipId, vm.text)
                .success(function (data) {
                    vm.text = '';
                    getMessages();

                    signalr.invoke('messageSentTo', data.toId);
                })
                .error(function (error) {
                    console.error(error);
                });
            }

            gaManager.sendMessage();
            analytics.action('send message');
        }

        accountManager.getUserInfo()
        .success(function (data) {

            vm.userInfo = angular.copy(data);

            friendshipsManager.getOne($stateParams.friendshipId)
            .success(function (data) {
                console.log('friends data', data);
                console.log('friends name', data.to.displayName);
                
                if (data.to.id === vm.userInfo.id) $rootScope.$emit('friendNameHasChanged', data.from.displayName);
                else $rootScope.$emit('friendNameHasChanged', data.to.displayName);
            });

            getMessages();
        });

        function getMessages() {
            messagesManager.getMessages($stateParams.friendshipId)
            .success(function (data) {
                vm.messages = _.map(data, function (item) {
                    if (item.createdDate) {
                        item.createdDate = moment(item.createdDate).format('MMM Do YYYY HH:mm:ss');
                    }

                    return item;
                });
                $ionicScrollDelegate.scrollBottom();

                vm.unseen = _.filter(vm.messages, function (item) {
                    return item.toId === vm.userInfo.id && !item.seen;
                });

                vm.unseen = _.map(vm.unseen, function (item) {
                    return item.id;
                });

                console.info('vm.messages', vm.messages);
                console.info('vm.unseen', vm.unseen);

                messagesManager.seen({
                    ids: vm.unseen
                }).success(function () {
                    messagesManager.getHasUnseenMessages();
                })
            })
            .error(function (error) {
                console.error(error);
            });
        }

    }])
    .directive('pressedEnter', function () {
        return function (scope, element, attrs) {
            element.bind("keydown keypress", function (event) {
                if (event.which === 13) {
                    scope.$apply(function () {
                        scope.$eval(attrs.pressedEnter);
                    });

                    event.preventDefault();
                }
            });
        };
    });

}());
