﻿(function () {
    angular.module('app').controller('chatCtrl', ['$rootScope', '$state', '$timeout', '$interval', '$ionicLoading', '$ionicPopup',
        'signalr', 'qbManager', 'accountManager', 'quizzManager', 'friendshipsManager', 'analytics', 'gaManager',
        function ($rootScope, $state, $timeout, $interval, $ionicLoading, $ionicPopup, signalr, qbManager, accountManager, quizzManager, friendshipsManager, analytics, gaManager) {
            var vm = this;

            analytics.pageActivated('chat');

            var options = true;
            var blur = false;

            var reportedId = null;
            var next = true;

            var peerConnection = null;

            var superHardCodedBoolean = false;

            var timeout = null;
            var interval = null;

            vm.inFriends = false;
            vm.isBrowser = ionic.Platform.is('browser') || typeof cordova == "undefined";
            vm.cameraSources = [];
            vm.showTutorial = localStorage['showTutorial'];

            getCameraSources(init);

            function getCameraSources(callback) {

                navigator.mediaDevices.enumerateDevices().then(function (devices) {
                    console.info('devices', devices);

                    vm.cameraSources = _.filter(devices, function (item) {
                        return item.kind === 'videoinput';
                    });
                    console.info('vm.cameraSources', vm.cameraSources);
                    vm.cameraSources = _.map(vm.cameraSources, function (item) {
                        return {
                            id: item.deviceId,
                            kind: item.kind,
                            label: item.label,
                            active: false
                        };
                    });
                    //if (vm.cameraSources.length > 1) vm.cameraSources[1].active = true;
                    if (vm.cameraSources.length > 0) vm.cameraSources[0].active = true;
                    console.info('vm.cameraSources', vm.cameraSources);
                    $rootScope.$apply();

                    if (callback) callback();

                });
            }

            function init() {
                vm.collapsedExpanded = false;
                vm.signalrSession = signalr.session;
                vm.callingSession = null;

                vm.qbUserId = null;

                vm.error = null;

                vm.disableNext = true;

                vm.timeleft = 0;

                vm.blur = !vm.blur;
                vm.options = !vm.options;


                vm.userInfo = accountManager.userInfo;
                $rootScope.$watch(function () {
                    return accountManager.userInfo;
                }, function () {
                    vm.userInfo = accountManager.userInfo;

                    if (vm.userInfo) {

                        if (!interval && !timeout) {
                            var freezeTime = vm.userInfo.gender == 'Female' ? 5000 : 10000;

                            vm.timeleft = freezeTime / 1000;
                            interval = $interval(function () {
                                if (vm.timeleft > 0) {
                                    vm.timeleft--;
                                } else {
                                    cancelInterval();
                                }
                            }, 1000);

                            timeout = $timeout(function () {
                                vm.disableNext = false;
                                cancelInterval();
                            }, freezeTime);
                        }

                    }
                });

                //$(window).on("beforeunload", function () {

                //    var confirmPopup = $ionicPopup.confirm({
                //        title: 'Do you really want to leave ?',
                //        template: 'Do you really want to leave ?'
                //    });

                //    confirmPopup.then(function (res) {
                //        if (res) {
                //            console.log('You are sure');
                //        } else {
                //            console.log('You are not sure');
                //        }
                //    });

                //    return 'Are you sure you want to leave?';
                //});


                accountManager.getUserInfo();

                vm.gettingCommmonInterests = false;
                vm.commonInterests = [];

                signalr.on('killSuccess', function () {
                    stop();
                });

                signalr.on('call', function (qbUserId) {
                    vm.qbUserId = angular.copy(qbUserId);
                    getFriendships();

                    var calleesIds = [qbUserId];
                    var sessionType = QB.webrtc.CallType.VIDEO;

                    vm.callingSession = QB.webrtc.createNewSession(calleesIds, sessionType);

                    setUserMedia(function (stream) {
                        vm.callingSession.call(null, function (error) {
                            if (error) {
                                console.error('vm.callingSession.call : error', error);
                                vm.error = JSON.stringify(error);
                                $rootScope.$apply();
                            } else {
                                _initRTC();
                                signalr.invoke('onCall');
                            }
                        });
                    });
                    $rootScope.$apply();
                });

                signalr.on('sessionHasChanged', function (signalrSession) {
                    if (signalrSession.secondsLeft === 0) {
                        signalrSession.secondsLeft = vm.signalrSession.secondsLeft;
                    }
                    vm.signalrSession = signalrSession;
                    $rootScope.$apply();
                    //console.info('chat sessionHasChanged', vm.signalrSession);
                });

                signalr.on('friendshipReceived', function (friendshipId) {
                    console.info(friendshipId);

                    var confirmPopup = $ionicPopup.confirm({
                        title: 'Accept Friend',
                        buttons: [
                            {
                                text: '<b>Accept</b>',
                                type: 'button-positive',
                                onTap: function (e) {
                                    $ionicLoading.show({
                                        template: 'Loading ...'
                                    });
                                    friendshipsManager.accept(friendshipId)
                                        .success(function () {
                                            $ionicLoading.hide();
                                            getFriendships();
                                            signalr.invoke('friendshipAccepted');
                                        })
                                        .error(function () {
                                            $ionicLoading.hide();
                                        });
                                }
                            },
                            {
                                text: 'Maybe Later'
                            }
                        ]
                    });

                });

                signalr.on('friendshipAccepted', function () {
                    $ionicLoading.show({
                        template: 'Your friend request has been accepted.',
                        duration: 1500
                    });
                });

                signalr.on('userInfoHasChanged', function () {
                    accountManager.getUserInfo();
                });

                signalr.on('sessionSDP', function (response) {
                    if (peerConnection) {
                        console.info('peerConnection', peerConnection);
                        var sdp = JSON.parse(response).sdp;
                        //obj.type = 'answer';
                        console.info('sdp', sdp);
                        //peerConnection.setRemoteDescription(new RTCSessionDescription(sdp));

                        if (sdp.type === "offer") {
                            peerConnection.setRemoteDescription(new RTCSessionDescription(sdp))
                                .then(function () {
                                    peerConnection.createAnswer().then(function (answer) {
                                        console.log('answer', answer);
                                        peerConnection.setLocalDescription(answer)
                                            .then(function () {
                                                signalr.invoke('sessionSDP', JSON.stringify({ "sdp": peerConnection.localDescription }));
                                            });
                                    });
                                });
                        } else {
                            peerConnection.setRemoteDescription(new RTCSessionDescription(sdp));
                        }

                        console.info(' ------------------------- sdp updated ------------------------- ');
                    }
                });

                signalr.invoke('waiting');

                QB.webrtc.onUserNotAnswerListener = function (session, userId) {
                };

                QB.webrtc.onCallListener = function (session, extension) {
                    if (session && !vm.callingSession) {
                        vm.callingSession = session;
                        getFriendships();
                        setUserMedia(function (stream) {
                            vm.callingSession.accept();
                            _initRTC();
                            signalr.invoke('onCall');
                        });
                        $rootScope.$apply();
                    }
                };

                QB.webrtc.onAcceptCallListener = function (session, userId, extension) {
                };

                QB.webrtc.onRejectCallListener = function (session, userId, extension) {
                    $rootScope.$apply();
                };

                QB.webrtc.onStopCallListener = function (session, userId, extension) {
                    stop();
                };

                QB.webrtc.onRemoteStreamListener = function (session, userId, remoteStream) {
                    // attach the remote stream to DOM element
                    vm.callingSession.attachMediaStream('remoteVideo', remoteStream);

                    //if (signalr.appHubProxy) signalr.appHubProxy.invoke('onCall');
                };

                QB.webrtc.onSessionConnectionStateChangedListener = function (session, userId, connectionState) {

                    if (session) {
                        if (session.state == 2) {
                            if (session.peerConnections) {
                                var peerConnections = session.peerConnections;
                                var peerConnection = null;
                                for (var k in peerConnections) {
                                    peerConnection = peerConnections[k];
                                    break;
                                }
                                if (peerConnection) {
                                    if (!vm.gettingCommmonInterests && vm.commonInterests.length == 0) {
                                        getCommonInterests(peerConnection.userID);
                                    }
                                }
                            }
                        }
                    }
                };

                vm.stop = function (n) {
                    if (n && vm.disableNext) return;
                    next = n;
                    signalr.invoke('stop', n, false);
                    console.info('stop', n);
                    gaManager.endChat();
                    analytics.action('stop chat', [{
                        data: JSON.stringify({
                            next: n
                        })
                    }]);
                }

                vm.report = function () {
                    console.info('vm.signalrSession', vm.signalrSession);
                    if (vm.signalrSession && vm.signalrSession.opponentSession && vm.signalrSession.opponentSession.user) {
                        reportedId = vm.signalrSession.opponentSession.user.id;
                    }
                    signalr.invoke('stop', false, false);

                    analytics.action('go to report');
                }

                vm.addTime = function () {
                    signalr.invoke('addTime');

                    analytics.action('add time');
                }

                vm.addFriend = function () {
                    console.info(vm.signalrSession);

                    $ionicLoading.show({
                        template: 'Loading ...'
                    });
                    friendshipsManager.create(vm.signalrSession.opponentSession.user.id)
                        .success(function (data) {

                            $ionicLoading.hide();

                            $ionicLoading.show({
                                template: 'Invite sent.',
                                duration: 1500
                            });



                            signalr.invoke('frienshipSent', data.id);
                            vm.options = false;
                        })
                        .error(function () {

                            $ionicLoading.hide();

                            $ionicLoading.show({
                                template: 'An error occured.',
                                duration: 1500
                            });
                        });

                    gaManager.addFriend();

                    analytics.action('add friend');
                }

                vm.toggle = function () {
                    vm.blur = !vm.blur;
                    vm.options = !vm.options;
                    if (vm.blur) vm.collapsedExpanded = false;

                    analytics.action('toggle menu');

                    setUserMedia();
                };

                vm.getQuestionFromChoice = function (qcId) {
                    quizzManager.getQuestionFromChoice(qcId)
                        .success(function (data) {
                            if (data && data.length > 0) {
                                $ionicLoading.show({
                                    template: JSON.stringify(data),
                                    duration: 1500
                                });
                            }
                        });

                    analytics.action('show common interest question');
                }

                vm.changeCamera = function () {

                    var activeIndex;
                    for (activeIndex = 0; activeIndex < vm.cameraSources.length; activeIndex++) {
                        if (vm.cameraSources[activeIndex].active) break;
                    }

                    var nextIndex = activeIndex + 1 === vm.cameraSources.length ? 0 : activeIndex + 1;

                    for (var k = 0; k < vm.cameraSources.length; k++) {
                        if (k === nextIndex) vm.cameraSources[k].active = true;
                        else vm.cameraSources[k].active = false;
                    }

                    setUserMedia(function (stream) {

                        peerConnection.removeStream(peerConnection.getLocalStreams()[0]);
                        peerConnection.addLocalStream(stream);
                    });

                    analytics.action('change camera');
                }
            }

            function setUserMedia(callback) {

                var cameraSource = _.find(vm.cameraSources, function (item) {
                    return item.active;
                });

                console.info('cameraSource', cameraSource);

                var mediaParams = {
                    audio: true,
                    video: {
                        mandatory: { sourceId: cameraSource.id }
                    },
                    options: {
                        muted: true,
                        mirror: true
                    },
                    elemId: 'localVideo'
                };


                vm.callingSession.getUserMedia(mediaParams, function (error, stream) {
                    if (error) {
                        console.error('vm.callingSession.getUserMedia : error', error);
                        vm.error = JSON.stringify(error);
                        $rootScope.$apply();
                    } else {

                        console.info('stream', stream);
                        console.info('stream.getVideoTracks()', stream.getVideoTracks());

                        if (callback) callback(stream);
                    }
                });
            }

            function _initRTC() {

                if (!peerConnection) {
                    peerConnection = _getPeerConnection();

                    if (!peerConnection.onnegotiationneeded) {
                        peerConnection.onnegotiationneeded = function (e) {
                            console.info(' ---------------- onnegotiationneeded ---------------- ', e);

                            superHardCodedBoolean = !superHardCodedBoolean;

                            if (!superHardCodedBoolean) {
                                peerConnection.createOffer()
                                    .then(function (offer) {
                                        peerConnection.setLocalDescription(offer);
                                    })
                                    .then(function () {
                                        console.info('peerConnection.localDescription', peerConnection.localDescription);
                                        signalr.invoke('sessionSDP', JSON.stringify({ "sdp": peerConnection.localDescription }));
                                    });
                            }
                        }
                    }
                }
            }

            function _getPeerConnection() {
                var peerConnection = null;
                if (vm.callingSession.peerConnections) {
                    for (var k in vm.callingSession.peerConnections) {
                        peerConnection = vm.callingSession.peerConnections[k];
                        break;
                    }
                }

                return peerConnection;
            }

            function stop() {
                if (vm.callingSession) {

                    vm.callingSession.detachMediaStream();
                    vm.callingSession.stop();

                    QB.webrtc.sessions = {};

                    vm.callingSession = {};
                }

                qbManager.stop();

                console.info('reportedId', reportedId);
                if (reportedId) {
                    $state.go('app.report', { userId: reportedId });
                }
                else if (next) {
                    $state.go('matching');
                }
                else {
                    $state.go('app.zero', { instruction : 'openMenu'});
                }

                cancelTimeout();
                cancelInterval();
            }

            function getCommonInterests(qbOpponentId) {
                vm.gettingCommmonInterests = true;
                quizzManager.getCommonInterests(qbOpponentId)
                    .success(function (data) {
                        vm.commonInterests = angular.copy(data);

                        vm.gettingCommmonInterests = false;
                    })
                    .error(function (error) {
                        console.error(error);

                        vm.gettingCommmonInterests = false;
                    });
            }

            function cancelTimeout() {
                if (timeout) {
                    $timeout.cancel(timeout);
                    timeout = null;
                }
            }

            function cancelInterval() {
                if (interval) {
                    $interval.cancel(interval);
                    interval = null;
                }
            }

            function getFriendships() {
                friendshipsManager.getAll()
                .success(function (data) {
                    for (var i in data) {
                        if (data[i].id && data[i].from && data[i].from.id === vm.signalrSession.opponentSession.user.id) {
                            vm.inFriends = true;
                        };
                    };
                });
            }

        }])
        .directive('animateOnChange', function ($timeout) {
            return function (scope, element, attr) {
                scope.$watch(attr.animateOnChange, function (nv, ov) {
                    if (nv > ov) {
                        element.addClass('added-seconds');
                        $timeout(function () {
                            element.removeClass('added-seconds');
                        }, 1000);
                    }
                    //console.log(nv);

                    if ((nv< 10) && (nv!=0)) {
                        console.log(nv);
                        element.addClass('secondsleft');                        
                    }
                    else {
                        element.removeClass('secondsleft');                        
                    }

                    //else (nv > 9) {
                    //    element.removeClass('secondsleft');                        
                    //}

                    //if (nv < 10) {
                    //    $timeout(function () {
                    //        element.css('display': 'inline-block');
                    //    }, 1000);
                    //}


                    //if (nv === 4) {
                    //    element.addClass('secondsleft-4');
                    //    $timeout(function () {
                    //        element.removeClass('secondsleft-4');
                    //    }, 500);
                    //} else if (nv === 3) {
                    //    element.addClass('secondsleft-3');
                    //    $timeout(function () {
                    //        element.removeClass('secondsleft-3');
                    //    }, 500);
                    //} else if (nv === 2) {
                    //    element.addClass('secondsleft-2');
                    //    $timeout(function () {
                    //        element.removeClass('secondsleft-2');
                    //    }, 500);
                    //} else if (nv === 1) {
                    //    element.addClass('secondsleft-1');
                    //    $timeout(function () {
                    //        element.removeClass('secondsleft-1');
                    //    }, 500);
                    //}

                });
            };
        });
} ());