﻿(function () {
    angular.module('app').controller('matchingCtrl', ['$rootScope', '$scope', '$state', '$stateParams', '$window', '$timeout', '$interval', '$ionicLoading',
        'accountManager', 'quizzManager', 'signalr', 'qbManager', 'deviceManager', 'analytics', 'session', 'gaManager',
    function ($rootScope, $scope, $state, $stateParams, $window, $timeout, $interval, $ionicLoading, accountManager, quizzManager, signalr, qbManager, deviceManager, analytics, session, gaManager) {
        var vm = this;

        analytics.pageActivated('matching');
        
        // deviceManager.requestCameraAuthorization()
        //     .then(function () {
        //         deviceManager.requestMicrophoneAuthorization()
        //             .then(function () {
        //                 deviceManager.requestModifyAudioSettingsAuthorization()
        //                     .then()
        //                     .catch(function(error){
        //                         console.error(error);
        //                     });
        //             })
        //             .catch(function (error) {
        //                 console.error(error);
        //             });
        //     })
        //     .catch(function (error) {
        //         console.error(error);
        //     });

        var intervalPromise = null;

        $scope.$on(
          "$destroy",
          function (event) {
              if (intervalPromise) $interval.cancel(intervalPromise);
          }
        );

        signalr.on('startError', function (message) {
            console.error(message);
            vm.error = 'Failed to connect to start.';
        });

        signalr.on('connectToChat', function () {
            vm.log = 'Set...';
            
            qbManager.connect(function (session) {
                vm.log = 'Go!';
                signalr.invoke('connectedToChat');
            }, function () {
                console.error('Failed to connect to chat.');
                vm.error = 'Failed to connect to chat.';
            });
        });

        signalr.on('goToChat', function () {

            if (intervalPromise) $interval.cancel(intervalPromise);

            $state.go('chat');

        });

        signalr.on('timeout', function () {
            $ionicLoading.show({
                template: 'Try again in a few minutes.',
                duration: 1500
            });
        });

        signalr.on('noSecondsLeft', function () {
            $ionicLoading.show({
                template: 'You have no seconds left.',
                duration: 1500
            });
        });

        signalr.on('killSuccess', function () {
            qbManager.stop();
        });

        vm.questions = [];
        vm.index = 0;
        vm.question = null;

        vm.log = null;
        vm.error = null;

        if (accountManager && accountManager.userInfo && accountManager.userInfo.credits) {
            vm.credits = accountManager.userInfo.credits;
        }
        else {
            vm.credits = 0;
        }

        vm.showPlusCredits = false;

        vm.answerQuestion = function (choice) {

            vm.credits++;

            vm.showPlusCredits = true;
            $timeout(function () {
                vm.showPlusCredits = false;

                choice.isTheAnswer = true;

                if (vm.questions && vm.questions.length > vm.index) {
                    vm.question = vm.questions[vm.index++];
                }
                else {
                    vm.question = null;
                    vm.log = 'Sending you answers...';

                    quizzManager.answerQuestions({
                        questions: vm.questions
                    })
                      .success(function () {
                          signalr.invoke('start');
                          vm.log = 'Started...';
                          gaManager.startMatching();
                      })
                      .error(function () {
                          vm.error = 'Can not send your answers.';
                      });

                    setTimeout();

                }

            }, 1500);

            if (vm.index === 1) {
                gaManager.firstQuestion();
            } else if (vm.index === 2) {
                gaManager.secondQuestion();
            }

            analytics.action('answer question');
        }

        vm.stop = function () {
            signalr.invoke('stop', false, false);

            gaManager.endMatching();

            analytics.action('stop matching');

            $state.go('app.zero');
        }

        getQuestions();
        getGeolocation();

        function setTimeout() {

            var timeout = 30;

            vm.secondsLeft = angular.copy(timeout);
            intervalPromise = $interval(function () {
                
                if (vm.secondsLeft === 0) {
                    signalr.invoke('stop', false, true);
                    console.info('timeout invoked');

                    if (vm.questions.length) {
                        $state.go('app.zero', { instruction: 'goToMatching' });
                    }
                    else {
                        $state.go('app.zero');
                    }
                }

                vm.secondsLeft--;

            }, 1000, timeout + 1);
        }

        function getQuestions() {
            vm.log = 'Preparing questions for you... ';
            //return;
            quizzManager.getQuestions()
              .success(function (data) {

                  console.info('quizzManager.getQuestions()', data);

                  if (data.maxIn24h) {
                      if (!session.hasShownMaxIn24h) {
                          $ionicLoading.show({
                              template: 'No more questions for today.'
                          });

                          $timeout(function () {
                              $ionicLoading.hide();
                          }, 1500);

                          session.hasShownMaxIn24h = true;
                      }
                  }
                  else {
                      session.hasShownMaxIn24h = false;
                  }

                  vm.questions = angular.copy(data.questions);

                  if (!vm.questions || vm.questions.length === 0) setTimeout();

                  console.info('vm.questions', vm.questions);

                  if (vm.questions.length == 0) {
                      vm.log = 'No more question for you.';
                      $timeout(function () {
                          signalr.invoke('start');
                          vm.log = 'Ready...';
                          gaManager.startMatching();
                      }, 1000);
                      return;
                  }

                  if (vm.questions && vm.questions.length > vm.index) {
                      vm.question = vm.questions[vm.index++];
                  }
              })
              .error(function (error) {
                  vm.error = 'Can not get the questions.';
                  console.error(error);
              })
              .finally(function () {
                  vm.log = null;
              });
        }

        function getGeolocation() {
            deviceManager.geoLocation()
            .then(function (position) {
                console.info('position', position);
                accountManager.setLastLocation(position);
            })
            .catch(function (error) {
                console.error(error);
            });
        }

    }])
    .directive('animateOnChange', function ($timeout) {
        return function (scope, element, attr) {
            scope.$watch(attr.animateOnChange, function (nv, ov) {
                if (nv - ov === 1) {
                    element.addClass('added-credits');
                    $timeout(function () {
                        element.removeClass('added-credits');
                    }, 1000);
                }
            });
        };
    });

}());
